// MineSweeper.cpp: implementation of the CMineSweeper class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "UIClasses/win32mine.h"

#include "GraphicOutput.h"
#include "MineSweeper.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include <stdlib.h>
#include <time.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CMineSweeper::CMineSweeper(CGraphicOutput *pGO)
    : CActionHandler(pGO),
      m_height(9),
      m_width(9),
      m_mineCount(10),
      m_mineCountOriginal(10),
      m_running(ready),
      m_unOpenedCells(81)
{
}

CMineSweeper::~CMineSweeper()
{
}

void CMineSweeper::cellOpen(int irow, int icol)
{
    TRACE("cellOpen() irow=%d icol=%d\n", irow, icol);

    int i, j;

    if ((irow<0)||(irow>=m_height)||(icol<0)||(icol>=m_width))
        return;
    if ((m_openStatus[irow][icol] == opened) || 
        (m_openStatus[irow][icol] == flagged))
        return;
//    TRACE("%d %d\n", irow, icol);

    if (m_running == halt) 
        return;
    else if (m_running == ready)
    {
        m_running = running;
        m_pGO->startClock();
    }

    if (m_mines[irow][icol] < 0)
    {
        m_pGO->showBlastedMine(irow, icol);
        m_pGO->setRemainingMineCount(--m_mineCount);
        m_openStatus[irow][icol] = opened;
        m_pGO->stopClock();
        m_running = halt;
        m_pGO->showFrawn();
        for (i=0; i<m_height; i++)
            for (j=0; j<m_width; j++)
                if ((m_mines[i][j]<0)&&(m_openStatus[i][j]==unopened))
                {
                    m_pGO->showMine(i, j);
                    m_pGO->setRemainingMineCount(--m_mineCount);
                }
        return;
    }
    else if (m_mines[irow][icol] > 0)
    {
        m_pGO->showSurroundingMineCount(m_mines[irow][icol], irow, icol);
        m_openStatus[irow][icol] = opened;
        m_unOpenedCells--;
    }
    else
    {
        m_openStatus[irow][icol] = opened;
        m_unOpenedCells--;
        m_pGO->showSurroundingMineCount(0, irow, icol);
        for (i=-1; i<=1; i++)
            for (j=-1; j<=1; j++)
                if ((i!=0)||(j!=0))
                    cellOpen(irow+i, icol+j);
    }

    if (m_unOpenedCells == m_mineCountOriginal) // winning
    {
        m_pGO->stopClock();
        m_running = halt;
        for (i=0; i<m_height; i++)
            for (j=0; j<m_width; j++)
                if ((m_mines[i][j]<0)&&(m_openStatus[i][j]==unopened))
                {
                    m_pGO->showMine(i, j);
                    m_pGO->setRemainingMineCount(--m_mineCount);
                }
   }
}

void CMineSweeper::cellMark(int irow, int icol)
{
    TRACE("cellMark() irow=%d icol=%d\n", irow, icol);


    if (m_running == halt) 
        return;
    else if (m_running == ready)
    {
        m_running = running;
        m_pGO->startClock();
    }

    if ((m_openStatus[irow][icol] == opened))
        return;
    else
    {
        switch (m_openStatus[irow][icol])
        {
        case unopened:
            m_openStatus[irow][icol] = flagged;
            m_pGO->setRemainingMineCount(--m_mineCount);
            m_pGO->showFlag(irow, icol);
            break;
        case flagged:
            m_openStatus[irow][icol] = questioned;
            m_pGO->setRemainingMineCount(++m_mineCount);
            m_pGO->showQuestionMark(irow, icol);
            break;
        case questioned:
            m_openStatus[irow][icol] = unopened;
            m_pGO->showCell(irow, icol);
            break;
        }
    }
}

/* level 0: board_width = 9, board_height = 9, number_of_mines = 10 */
/* level 1: board_width = 16, board_height = 16, number_of_mines = 40 */
/* level 2: board_width = 30, board_height = 16, number_of_mines = 99 */

void CMineSweeper::newGame(int width, int height, int mineCount)
{
    TRACE("newGame() width=%d height=%d mineCount=%d\n", width, height, mineCount);

    int i, j;

    m_width = width;
    m_height = height;
    m_unOpenedCells = width * height;
    m_mineCountOriginal = mineCount;
    m_mineCount = m_mineCountOriginal;
    deployMines();
    m_running = ready;

    m_pGO->setRemainingMineCount(m_mineCount);
    m_pGO->showSmile();
    m_pGO->resetClock();
    for (i=0; i<m_height; i++)
    {
        for (j=0; j<m_width; j++)
        {
            m_openStatus[i][j] = unopened;
            m_pGO->showCell(i, j);
//            TRACE("%3d ", m_mines[i][j]);
/*            if (m_mines[i][j] < 0)
                m_pGO->showMine(i, j);
            else
                m_pGO->showSurroundingMineCount(m_mines[i][j], i, j);
*/
        }
//        TRACE("\n");
    }
}

void CMineSweeper::exitProgram()
{
    TRACE("exitProgram()\n");
}

void CMineSweeper::deployMines()
{
    int i, j, irow, icol, i0, j0, count;

    for (i=0; i<m_height; i++)
        for (j=0; j<m_width; j++)
        {
            m_mines[i][j] = 0;
        }

    for (i=0; i<m_mineCount; i++)
    {
        do 
        {
            irow = rand() % m_height;
            icol = rand() % m_width;
        } while (m_mines[irow][icol] < 0);
        m_mines[irow][icol] = -1;
    }

    for (i=0; i<m_height; i++)
        for (j=0; j<m_width; j++)
        {
            if (m_mines[i][j] == 0)
            {
                count = 0;
                for (i0=-1; i0<=1; i0++)
                {
                    if (i+i0 < 0) continue;
					if (i+i0 >= m_height) continue;
                    for (j0=-1; j0<=1; j0++)
                    {
                        if (j+j0 < 0) continue;
						if (j+j0 >= m_width) continue;
                        if ((i0==0) && (j0==0)) continue;
                        if (m_mines[i+i0][j+j0] == -1)
                            count++;
                    }
                }
                m_mines[i][j] = count;
//                TRACE("%d %d %d\n", i, j, m_mines[i][j]);
            }
        }
}
